"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ngAdd = void 0;
const core_1 = require("@angular-devkit/core");
const schematics_1 = require("@angular-devkit/schematics");
const schematics_2 = require("@angular/cdk/schematics");
const tasks_1 = require("@angular-devkit/schematics/tasks");
const dependencies_1 = require("@schematics/angular/utility/dependencies");
const workspace_1 = require("@schematics/angular/utility/workspace");
const change_1 = require("@schematics/angular/utility/change");
const workspace_models_1 = require("@schematics/angular/utility/workspace-models");
const ng_cli_version_1 = require("../util/ng-cli-version");
const has_tailwind_support_1 = require("../util/has-tailwind-support");
const NGX_BUILD_PLUS_BUILDER_TARGET = 'ngx-build-plus:browser';
const NGX_BUILD_PLUS_DEV_BUILDER_TARGET = 'ngx-build-plus:dev-server';
const NGX_BUILD_PLUS_KARMA_BUILDER_TARGET = 'ngx-build-plus:karma';
function ngAdd(_options) {
    return (host) => __awaiter(this, void 0, void 0, function* () {
        const workspace = yield workspace_1.getWorkspace(host);
        const project = schematics_2.getProjectFromWorkspace(workspace, _options.project);
        const projectName = _options.project || Object.keys(workspace.projects)[0];
        if (!project) {
            throw new schematics_1.SchematicsException(`Project ${projectName} is not defined in this workspace.`);
        }
        // FIXME project.projectType not available anymore with angular 11
        // if (project.projectType !== 'application') {
        //   throw new SchematicsException(
        //     `ngx-tailwind requires a project type of "application" but ${projectName} isn't.`,
        //   );
        // }
        const tailwindPlugins = _options.tailwindPlugins || [];
        const tailwindPluginDependencies = tailwindPlugins === null || tailwindPlugins === void 0 ? void 0 : tailwindPlugins.map((plugin) => `@tailwindcss/${plugin}`);
        const requireTailwindPlugins = tailwindPluginDependencies.map((plugin) => `require('${plugin}')\n`);
        const cliVersion = ng_cli_version_1.getCliVersionAsNumber(host);
        if (has_tailwind_support_1.hasTailwindSupport(host)) {
            return schematics_1.chain([
                addDependenciesWithTailwindSupport(_options),
                addTailwindPlugins(tailwindPluginDependencies),
                addNpmScripts(_options, cliVersion),
                updateStyles(_options, workspace),
                generateTailwindConfig(_options, requireTailwindPlugins),
                install(),
            ]);
        }
        else {
            return schematics_1.chain([
                addDependenciesBeforeTailwindSupport(_options),
                addTailwindPlugins(tailwindPluginDependencies),
                addNpmScripts(_options, cliVersion),
                updateStyles(_options, workspace),
                generateTailwindAndWebpackConfig(_options, requireTailwindPlugins),
                updateAngularJSON(_options, workspace),
                install(),
            ]);
        }
    });
}
exports.ngAdd = ngAdd;
function addDependenciesWithTailwindSupport(_options) {
    return (host) => {
        dependencies_1.addPackageJsonDependency(host, {
            type: dependencies_1.NodeDependencyType.Dev,
            name: 'tailwindcss',
            version: _options.tailwindVersion,
        });
        dependencies_1.addPackageJsonDependency(host, {
            type: dependencies_1.NodeDependencyType.Dev,
            name: 'autoprefixer',
            version: _options.autoprefixerVersion,
        });
        dependencies_1.addPackageJsonDependency(host, {
            type: dependencies_1.NodeDependencyType.Dev,
            name: 'postcss',
            version: _options.postcssVersion,
        });
        if (!_options.disableCrossPlatform) {
            dependencies_1.addPackageJsonDependency(host, {
                type: dependencies_1.NodeDependencyType.Dev,
                name: 'cross-env',
                version: _options.crossEnvVersion,
            });
        }
    };
}
function addDependenciesBeforeTailwindSupport(_options) {
    return (host) => {
        dependencies_1.addPackageJsonDependency(host, {
            type: dependencies_1.NodeDependencyType.Dev,
            name: 'tailwindcss',
            version: _options.tailwindVersion,
        });
        dependencies_1.addPackageJsonDependency(host, {
            type: dependencies_1.NodeDependencyType.Dev,
            name: 'autoprefixer',
            version: _options.autoprefixerVersion,
        });
        dependencies_1.addPackageJsonDependency(host, {
            type: dependencies_1.NodeDependencyType.Dev,
            name: 'postcss',
            version: _options.postcssVersion,
        });
        if (_options.cssFormat === 'scss') {
            dependencies_1.addPackageJsonDependency(host, {
                type: dependencies_1.NodeDependencyType.Dev,
                name: 'postcss-scss',
                version: _options.postcssScssVersion,
            });
        }
        dependencies_1.addPackageJsonDependency(host, {
            type: dependencies_1.NodeDependencyType.Dev,
            name: 'postcss-import',
            version: _options.postcssImportVersion,
        });
        dependencies_1.addPackageJsonDependency(host, {
            type: dependencies_1.NodeDependencyType.Dev,
            name: 'postcss-loader',
            version: _options.postcssLoaderVersion,
        });
        dependencies_1.addPackageJsonDependency(host, {
            type: dependencies_1.NodeDependencyType.Dev,
            name: 'ngx-build-plus',
            version: _options.ngxBuildPlusVersion,
        });
        if (!_options.disableCrossPlatform) {
            dependencies_1.addPackageJsonDependency(host, {
                type: dependencies_1.NodeDependencyType.Dev,
                name: 'cross-env',
                version: _options.crossEnvVersion,
            });
        }
    };
}
function addTailwindPlugins(tailwindPlugins) {
    return (tree) => {
        tailwindPlugins.forEach((plugin) => dependencies_1.addPackageJsonDependency(tree, {
            type: dependencies_1.NodeDependencyType.Dev,
            name: plugin,
            version: 'latest',
        }));
    };
}
function updateStyles(options, workspace) {
    return (tree, context) => {
        const project = schematics_2.getProjectFromWorkspace(workspace, options.project);
        const stylePath = schematics_2.getProjectStyleFile(project, options.cssFormat);
        if (!stylePath) {
            context.logger.error(`Cannot update project styles file: Style path not found`);
            return tree;
        }
        const insertion = new change_1.InsertChange(stylePath, 0, options.cssFormat === 'css'
            ? getTailwindDirectives()
            : getTailwindImports());
        const recorder = tree.beginUpdate(stylePath);
        recorder.insertLeft(0, insertion.toAdd);
        tree.commitUpdate(recorder);
        return tree;
    };
}
/**
 * Used for css stylesheets
 */
function getTailwindDirectives() {
    return `@tailwind 'tailwindcss/base';\n
@tailwind 'tailwindcss/components';\n
@tailwind 'tailwindcss/utilities';\n`;
}
/**
 * Used for scss stylesheets
 */
function getTailwindImports() {
    return `@import 'tailwindcss/base';\n
@import 'tailwindcss/components';\n
@import 'tailwindcss/utilities';\n`;
}
/**
 * Generate webpack and tailwind config.
 *
 * @param options
 */
function generateTailwindConfig(options, requireTailwindPlugins) {
    return (_host) => __awaiter(this, void 0, void 0, function* () {
        const sourceTemplates = schematics_1.url(`./templates/tailwind`);
        const sourceParametrizedTemplates = schematics_1.apply(sourceTemplates, [
            schematics_1.template(Object.assign(Object.assign(Object.assign({}, options), { requireTailwindPlugins: requireTailwindPlugins }), core_1.strings)),
        ]);
        return schematics_1.mergeWith(sourceParametrizedTemplates);
    });
}
/**
 * Generate webpack and tailwind config.
 *
 * @param options
 */
function generateTailwindAndWebpackConfig(options, requireTailwindPlugins) {
    return (_host) => __awaiter(this, void 0, void 0, function* () {
        const sourceTemplates = schematics_1.url(`./templates/configs`);
        const sourceParametrizedTemplates = schematics_1.apply(sourceTemplates, [
            schematics_1.template(Object.assign(Object.assign(Object.assign({}, options), { requireTailwindPlugins: requireTailwindPlugins }), core_1.strings)),
        ]);
        return schematics_1.mergeWith(sourceParametrizedTemplates);
    });
}
function updateAngularJSON(options, workspace) {
    return (tree, context) => {
        const project = schematics_2.getProjectFromWorkspace(workspace, options.project);
        const browserTargets = schematics_2.getTargetsByBuilderName(project, workspace_models_1.Builders.Browser);
        const devServerTargets = schematics_2.getTargetsByBuilderName(project, workspace_models_1.Builders.DevServer);
        const karmaServerTargets = schematics_2.getTargetsByBuilderName(project, workspace_models_1.Builders.Karma);
        browserTargets.forEach((browserTarget) => {
            browserTarget.builder = NGX_BUILD_PLUS_BUILDER_TARGET;
            browserTarget.options = Object.assign({ extraWebpackConfig: 'webpack.config.js' }, browserTarget.options);
        });
        devServerTargets.forEach((browserTarget) => {
            browserTarget.builder = NGX_BUILD_PLUS_DEV_BUILDER_TARGET;
            browserTarget.options = Object.assign({ extraWebpackConfig: 'webpack.config.js' }, browserTarget.options);
        });
        karmaServerTargets.forEach((browserTarget) => {
            browserTarget.builder = NGX_BUILD_PLUS_KARMA_BUILDER_TARGET;
            browserTarget.options = Object.assign({ extraWebpackConfig: 'webpack.config.js' }, browserTarget.options);
        });
        return workspace_1.updateWorkspace(workspace)(tree, context);
    };
}
function addNpmScripts(_options, cliVersion) {
    return (tree) => {
        const pkgPath = 'package.json';
        const buffer = tree.read(pkgPath);
        if (buffer === null) {
            throw new schematics_1.SchematicsException('Could not find package.json');
        }
        const pkg = JSON.parse(buffer.toString());
        let prodFlag = '--prod';
        if (cliVersion >= 12) {
            prodFlag = '--configuration production';
        }
        if (_options.disableCrossPlatform) {
            pkg.scripts['build:prod'] = `NODE_ENV=production ng build ${prodFlag}`;
        }
        else {
            pkg.scripts['build:prod'] = `cross-env NODE_ENV=production ng build ${prodFlag}`;
        }
        tree.overwrite(pkgPath, JSON.stringify(pkg, null, 2));
        return tree;
    };
}
function install() {
    return (_host, context) => __awaiter(this, void 0, void 0, function* () {
        // Install the dependency
        context.addTask(new tasks_1.NodePackageInstallTask());
    });
}
//# sourceMappingURL=index.js.map